from app.models import User
from flask_wtf import FlaskForm
from wtforms import (BooleanField, PasswordField, StringField, SubmitField,
                     ValidationError)
from wtforms.validators import DataRequired, Email, EqualTo, Length, Regexp


class LoginForm(FlaskForm):
    user = StringField('Email address or username', validators=[DataRequired(), Length(1, 64)])
    password = PasswordField('Password', validators=[DataRequired()])
    remember_me = BooleanField('Keep me logged in')
    submit = SubmitField('Log In')


class PasswordResetForm(FlaskForm):
    password = PasswordField(
        'New Password',
        validators=[DataRequired(),
                    EqualTo('password2', message='Passwords must match')]
    )
    password2 = PasswordField(
        'Confirm password',
        validators=[DataRequired(),
                    EqualTo('password', message='Passwords must match.')]
    )
    submit = SubmitField('Reset Password')


class PasswordResetRequestForm(FlaskForm):
    email = StringField('Email', validators=[DataRequired(), Email()])
    submit = SubmitField('Reset Password')


class RegistrationForm(FlaskForm):
    email = StringField('Email', validators=[DataRequired(), Email()])
    username = StringField(
        'Username',
        validators=[DataRequired(), Length(1, 64),
                    Regexp('^[A-Za-z][A-Za-z0-9_.]*$', 0,
                           'Usernames must have only letters, numbers, dots '
                           'or underscores')]
    )
    password = PasswordField(
        'Password',
        validators=[DataRequired(),
                    EqualTo('password2', message='Passwords must match.')]
    )
    password2 = PasswordField('Confirm password', validators=[DataRequired()])
    submit = SubmitField('Register')

    def validate_email(self, field):
        if User.query.filter_by(email=field.data.lower()).first():
            raise ValidationError('Email already registered.')

    def validate_username(self, field):
        if User.query.filter_by(username=field.data).first():
            raise ValidationError('Username already in use.')
